@@ -6,7 +6,6 @@ import os
 import enum
 import numpy
 import struct
-import logging
 import datetime
 
 from . import base
@@ -20,9 +19,6 @@ except ImportError:  # pragma: no cover
     _speedups = None
 
 
-logger = logging.getLogger(__name__)
-
-
 class Mode(enum.IntEnum):
     #: Automatically detect whether the output is a TTY, if so, write ASCII
     #: otherwise write BINARY
@@ -47,6 +43,8 @@ HEADER_SIZE = 80
 COUNT_SIZE = 4
 #: The maximum amount of triangles we can read from binary files
 MAX_COUNT = 1e8
+#: The header format, can be safely monkeypatched. Limited to 80 characters
+HEADER_FORMAT = '{package_name} ({version}) {now} {name}'
 
 
 class BaseStl(base.BaseMesh):
@@ -60,7 +58,7 @@ class BaseStl(base.BaseMesh):
         :param file fh: The file handle to open
         :param int mode: Automatically detect the filetype or force binary
         '''
-        header = fh.read(HEADER_SIZE).lower()
+        header = fh.read(HEADER_SIZE)
         if not header:
             return
 
@@ -69,11 +67,15 @@ class BaseStl(base.BaseMesh):
 
         name = ''
 
-        if mode in (AUTOMATIC, ASCII) and header.startswith(b('solid')):
+        if mode in (AUTOMATIC, ASCII) and header[:5].lower() == b('solid'):
             try:
                 name, data = cls._load_ascii(
                     fh, header, speedups=speedups)
             except RuntimeError as exception:
+                # Disable fallbacks in ASCII mode
+                if mode is ASCII:
+                    raise
+
                 (recoverable, e) = exception.args
                 # If we didn't read beyond the header the stream is still
                 # readable through the binary reader
@@ -123,7 +125,12 @@ class BaseStl(base.BaseMesh):
         name = header.strip()
 
         # Read the rest of the binary data
-        return name, numpy.fromfile(fh, dtype=cls.dtype, count=count)
+        try:
+            return name, numpy.fromfile(fh, dtype=cls.dtype, count=count)
+        except io.UnsupportedOperation:
+            data = numpy.frombuffer(fh.read(), dtype=cls.dtype, count=count)
+            # Copy to make the buffer writable
+            return name, data.copy()
 
     @classmethod
     def _ascii_reader(cls, fh, header):
@@ -136,20 +143,22 @@ class BaseStl(base.BaseMesh):
         lines = b(header).split(b('\n'))
 
         def get(prefix=''):
-            prefix = b(prefix)
+            prefix = b(prefix).lower()
 
             if lines:
-                line = lines.pop(0)
+                raw_line = lines.pop(0)
             else:
                 raise RuntimeError(recoverable[0], 'Unable to find more lines')
+
             if not lines:
                 recoverable[0] = False
 
                 # Read more lines and make sure we prepend any old data
                 lines[:] = b(fh.read(BUFFER_SIZE)).split(b('\n'))
-                line += lines.pop(0)
+                raw_line += lines.pop(0)
 
-            line = line.lower().strip()
+            raw_line = raw_line.strip()
+            line = raw_line.lower()
             if line == b(''):
                 return get(prefix)
 
@@ -158,15 +167,17 @@ class BaseStl(base.BaseMesh):
                     values = line.replace(prefix, b(''), 1).strip().split()
                 elif line.startswith(b('endsolid')):
                     # go back to the beginning of new solid part
-                    size_unprocessedlines = sum(len(l) + 1 for l in lines) - 1
+                    size_unprocessedlines = sum(
+                        len(line) + 1 for line in lines) - 1
+
                     if size_unprocessedlines > 0:
                         position = fh.tell()
                         fh.seek(position - size_unprocessedlines)
                     raise StopIteration()
                 else:
-                    raise RuntimeError(recoverable[0],
-                                       '%r should start with %r' % (line,
-                                                                    prefix))
+                    raise RuntimeError(
+                        recoverable[0],
+                        '%r should start with %r' % (line, prefix))
 
                 if len(values) == 3:
                     return [float(v) for v in values]
@@ -174,15 +185,9 @@ class BaseStl(base.BaseMesh):
                     raise RuntimeError(recoverable[0],
                                        'Incorrect value %r' % line)
             else:
-                return b(line)
+                return b(raw_line)
 
         line = get()
-        if not line.startswith(b('solid ')) and line.startswith(b('solid')):
-            cls.warning('ASCII STL files should start with solid <space>. '
-                        'The application that produced this STL file may be '
-                        'faulty, please report this error. The erroneous '
-                        'line: %r', line)
-
         if not lines:
             raise RuntimeError(recoverable[0],
                                'No lines found, impossible to read')
@@ -210,11 +215,18 @@ class BaseStl(base.BaseMesh):
             except AssertionError as e:  # pragma: no cover
                 raise RuntimeError(recoverable[0], e)
             except StopIteration:
-                raise
+                return
 
     @classmethod
     def _load_ascii(cls, fh, header, speedups=True):
-        if _speedups and speedups:
+        # Speedups does not support non file-based streams
+        try:
+            fh.fileno()
+        except io.UnsupportedOperation:
+            speedups = False
+        # The speedups module is covered by travis but it can't be tested in
+        # all environments, this makes coverage checks easier
+        if _speedups and speedups:  # pragma: no cover
             return _speedups.ascii_read(fh, header)
         else:
             iterator = cls._ascii_reader(fh, header)
@@ -259,7 +271,7 @@ class BaseStl(base.BaseMesh):
             pass
 
     def _write_ascii(self, fh, name):
-        if _speedups and self.speedups:
+        if _speedups and self.speedups:  # pragma: no cover
             _speedups.ascii_write(fh, b(name), self.data)
         else:
             def p(s, file):
@@ -279,17 +291,20 @@ class BaseStl(base.BaseMesh):
 
             p('endsolid %s' % name, file=fh)
 
-    def _write_binary(self, fh, name):
-        # Create the header
-        header = '%s (%s) %s %s' % (
-            metadata.__package_name__,
-            metadata.__version__,
-            datetime.datetime.now(),
-            name,
+    def get_header(self, name):
+        # Format the header
+        header = HEADER_FORMAT.format(
+            package_name=metadata.__package_name__,
+            version=metadata.__version__,
+            now=datetime.datetime.now(),
+            name=name,
         )
 
         # Make it exactly 80 characters
-        header = header[:80].ljust(80, ' ')
+        return header[:80].ljust(80, ' ')
+
+    def _write_binary(self, fh, name):
+        header = self.get_header(name)
         packed = struct.pack(s('<i'), self.data.size)
 
         if isinstance(fh, io.TextIOWrapper):  # pragma: no cover
@@ -315,35 +330,32 @@ class BaseStl(base.BaseMesh):
         :param str filename: The file to load
         :param bool calculate_normals: Whether to update the normals
         :param file fh: The file handle to open
-        :param dict \**kwargs: The same as for :py:class:`stl.mesh.Mesh`
+        :param dict kwargs: The same as for :py:class:`stl.mesh.Mesh`
 
         '''
         if fh:
             name, data = cls.load(
                 fh, mode=mode, speedups=speedups)
         else:
-            try:
-                with open(filename, 'rb') as fh:
-                    name, data = cls.load(
-                        fh, mode=mode, speedups=speedups)
-            except AssertionError:  # pragma: no cover
-                logger.warn('Unable to read the file with speedups, retrying')
-                with open(filename, 'rb') as fh:
-                    name, data = cls.load(
-                        fh, mode=Mode.ASCII, speedups=False)
+            with open(filename, 'rb') as fh:
+                name, data = cls.load(
+                    fh, mode=mode, speedups=speedups)
 
         return cls(data, calculate_normals, name=name,
                    speedups=speedups, **kwargs)
 
     @classmethod
     def from_multi_file(cls, filename, calculate_normals=True, fh=None,
-                        mode=ASCII, speedups=True, **kwargs):
+                        mode=Mode.ASCII, speedups=True, **kwargs):
         '''Load multiple meshes from a STL file
 
+        Note: mode is hardcoded to ascii since binary stl files do not support
+        the multi format
+
         :param str filename: The file to load
         :param bool calculate_normals: Whether to update the normals
         :param file fh: The file handle to open
-        :param dict \**kwargs: The same as for :py:class:`stl.mesh.Mesh`
+        :param dict kwargs: The same as for :py:class:`stl.mesh.Mesh`
         '''
         if fh:
             close = False
@@ -357,13 +369,38 @@ class BaseStl(base.BaseMesh):
                 name, data = raw_data
                 yield cls(data, calculate_normals, name=name,
                           speedups=speedups, **kwargs)
-                raw_data = cls.load(fh, mode=mode,
+                raw_data = cls.load(fh, mode=ASCII,
                                     speedups=speedups)
 
         finally:
             if close:
                 fh.close()
 
+    @classmethod
+    def from_files(cls, filenames, calculate_normals=True, mode=Mode.AUTOMATIC,
+                   speedups=True, **kwargs):
+        '''Load multiple meshes from a STL file
+
+        Note: mode is hardcoded to ascii since binary stl files do not support
+        the multi format
+
+        :param list(str) filenames: The files to load
+        :param bool calculate_normals: Whether to update the normals
+        :param file fh: The file handle to open
+        :param dict kwargs: The same as for :py:class:`stl.mesh.Mesh`
+        '''
+        meshes = []
+        for filename in filenames:
+            meshes.append(cls.from_file(
+                filename,
+                calculate_normals=calculate_normals,
+                mode=mode,
+                speedups=speedups,
+                **kwargs))
+
+        data = numpy.concatenate([mesh.data for mesh in meshes])
+        return cls(data, calculate_normals=calculate_normals, **kwargs)
+
 
 StlMesh = BaseStl.from_file
 
